"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const faye_1 = require("faye");
const core_1 = require("@salesforce/core");
const i18n_1 = require("../i18n");
const TEST_RESULT_CHANNEL = '/systemTopic/TestResult';
const DEFAULT_STREAMING_TIMEOUT_MS = 14400;
class StreamingClient {
    constructor(connection) {
        this.apiVersion = '36.0';
        this.conn = connection;
        const streamUrl = this.getStreamURL(this.conn.instanceUrl);
        this.client = new faye_1.Client(streamUrl, {
            timeout: DEFAULT_STREAMING_TIMEOUT_MS
        });
        this.client.on('transport:up', () => {
            console.log(i18n_1.nls.localize('streaming_transport_up'));
        });
        this.client.on('transport:down', () => {
            console.log(i18n_1.nls.localize('streaming_transport_down'));
        });
        this.client.addExtension({
            incoming: (message, callback) => {
                if (message && message.channel === '/meta/handshake' && message.error) {
                    throw new Error(i18n_1.nls.localize('streaming_handshake_fail', message.error));
                }
                callback(message);
            }
        });
    }
    removeTrailingSlashURL(instanceUrl) {
        return instanceUrl ? instanceUrl.replace(/\/+$/, '') : '';
    }
    getStreamURL(instanceUrl) {
        const urlElements = [
            this.removeTrailingSlashURL(instanceUrl),
            'cometd',
            this.apiVersion
        ];
        return urlElements.join('/');
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            const username = this.conn.getUsername();
            const org = yield core_1.Org.create({ aliasOrUsername: username });
            yield org.refreshAuth();
            const accessToken = this.conn.getConnectionOptions().accessToken;
            if (accessToken) {
                this.client.setHeader('Authorization', `OAuth ${accessToken}`);
            }
            else {
                throw new Error(i18n_1.nls.localize('no_access_token_found'));
            }
        });
    }
    subscribe(testRunId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.subscribedTestRunId = testRunId;
            return new Promise((subscriptionResolve, subscriptionReject) => {
                try {
                    this.client.subscribe(TEST_RESULT_CHANNEL, (message) => __awaiter(this, void 0, void 0, function* () {
                        const result = yield this.handler(message);
                        if (result) {
                            this.client.disconnect();
                            subscriptionResolve(result);
                        }
                    }));
                }
                catch (e) {
                    this.client.disconnect();
                    subscriptionReject(e);
                }
            });
        });
    }
    isValidTestRunID(testRunId) {
        if (testRunId.length !== 15 && testRunId.length !== 18) {
            return false;
        }
        const testRunId15char = testRunId.substring(0, 14);
        const subscribedTestRunId15char = this.subscribedTestRunId.substring(0, 14);
        return subscribedTestRunId15char === testRunId15char;
    }
    handler(message) {
        return __awaiter(this, void 0, void 0, function* () {
            const testRunId = message.sobject.Id;
            if (!this.isValidTestRunID(testRunId)) {
                return null;
            }
            const queryApexTestQueueItem = `SELECT Id, Status, ApexClassId, TestRunResultId FROM ApexTestQueueItem WHERE ParentJobId = '${testRunId}'`;
            let completedRecordProcess = true;
            const result = (yield this.conn.tooling.query(queryApexTestQueueItem));
            if (result.records.length === 0) {
                throw new Error(i18n_1.nls.localize('no_test_queue_results', testRunId));
            }
            for (let i = 0; i < result.records.length; i++) {
                const item = result.records[i];
                if (item.Status === "Queued" /* Queued */ ||
                    item.Status === "Holding" /* Holding */ ||
                    item.Status === "Preparing" /* Preparing */ ||
                    item.Status === "Processing" /* Processing */) {
                    completedRecordProcess = false;
                    break;
                }
            }
            if (completedRecordProcess) {
                return result;
            }
            else {
                console.log(i18n_1.nls.localize('streaming_processing_test_run', testRunId));
            }
            return null;
        });
    }
}
exports.StreamingClient = StreamingClient;
//# sourceMappingURL=streamingClient.js.map