"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util = require("util");
const i18n_1 = require("../i18n");
const streaming_1 = require("../streaming");
class TestService {
    constructor(connection) {
        this.connection = connection;
    }
    runTestSynchronous(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.connection.tooling._baseUrl()}/runTestsSynchronous`;
            const request = {
                method: 'POST',
                url,
                body: JSON.stringify(options),
                headers: { 'content-type': 'application/json' }
            };
            const testRun = yield this.connection.tooling.request(request);
            return testRun;
        });
    }
    runTestAsynchronous(options, codeCoverage = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const sClient = new streaming_1.StreamingClient(this.connection);
            yield sClient.init();
            const url = `${this.connection.tooling._baseUrl()}/runTestsAsynchronous`;
            const request = {
                method: 'POST',
                url,
                body: JSON.stringify(options),
                headers: { 'content-type': 'application/json' }
            };
            const testRunId = (yield this.connection.tooling.request(request));
            const testQueueResult = yield sClient.subscribe(testRunId);
            return yield this.getTestResultData(testQueueResult, testRunId, codeCoverage);
        });
    }
    calculatePercentage(dividend, divisor) {
        let percentage = '0%';
        if (dividend > 0) {
            const calcPct = ((dividend / divisor) * 100).toFixed();
            percentage = `${calcPct}%`;
        }
        return percentage;
    }
    getTestResultData(testQueueResult, testRunId, codeCoverage = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let testRunSummaryQuery = 'SELECT AsyncApexJobId, Status, ClassesCompleted, ClassesEnqueued, ';
            testRunSummaryQuery +=
                'MethodsEnqueued, StartTime, EndTime, TestTime, UserId ';
            testRunSummaryQuery += `FROM ApexTestRunResult WHERE AsyncApexJobId = '${testRunId}'`;
            const testRunSummaryResults = (yield this.connection.tooling.query(testRunSummaryQuery));
            if (testRunSummaryResults.records.length === 0) {
                throw new Error(i18n_1.nls.localize('no_test_result_summary', testRunId));
            }
            const summaryRecord = testRunSummaryResults.records[0];
            let apexTestResultQuery = 'SELECT Id, QueueItemId, StackTrace, Message, ';
            apexTestResultQuery +=
                'RunTime, TestTimestamp, AsyncApexJobId, MethodName, Outcome, ApexLogId, ';
            apexTestResultQuery +=
                'ApexClass.Id, ApexClass.Name, ApexClass.NamespacePrefix, ApexClass.FullName ';
            apexTestResultQuery += 'FROM ApexTestResult WHERE QueueItemId IN (%s)';
            // TODO: this needs to iterate and create a comma separated string of ids
            // and check for query length
            const apexResultId = testQueueResult.records[0].Id;
            const apexTestResults = (yield this.connection.tooling.query(util.format(apexTestResultQuery, `'${apexResultId}'`)));
            let globalTestPassed = 0;
            let globalTestFailed = 0;
            let globalTestSkipped = 0;
            // Iterate over test results, format and add them as results.tests
            const testResults = [];
            apexTestResults.records.forEach(item => {
                switch (item.Outcome) {
                    case "Pass" /* Pass */:
                        globalTestPassed++;
                        break;
                    case "Fail" /* Fail */:
                    case "CompileFail" /* CompileFail */:
                        globalTestFailed++;
                        break;
                    case "Skip" /* Skip */:
                        globalTestSkipped++;
                        break;
                }
                testResults.push({
                    id: item.Id,
                    queueItemId: item.QueueItemId,
                    stackTrace: item.StackTrace,
                    message: item.Message,
                    asyncApexJobId: item.AsyncApexJobId,
                    methodName: item.MethodName,
                    outcome: item.Outcome,
                    apexLogId: item.ApexLogId,
                    apexClass: {
                        id: item.ApexClass.Id,
                        name: item.ApexClass.Name,
                        namespacePrefix: item.ApexClass.NamespacePrefix,
                        fullName: item.ApexClass.FullName
                    },
                    runTime: item.RunTime,
                    testTimestamp: item.TestTimestamp,
                    fullName: `${item.ApexClass.FullName}.${item.MethodName}`
                });
            });
            const result = {
                summary: {
                    failRate: this.calculatePercentage(globalTestFailed, testResults.length),
                    numTestsRan: testResults.length,
                    orgId: this.connection.getAuthInfoFields().orgId,
                    outcome: summaryRecord.Status,
                    passRate: this.calculatePercentage(globalTestPassed, testResults.length),
                    skipRate: this.calculatePercentage(globalTestSkipped, testResults.length),
                    testStartTime: summaryRecord.StartTime,
                    testExecutionTime: summaryRecord.TestTime,
                    testRunId,
                    userId: summaryRecord.UserId,
                    username: this.connection.getUsername()
                },
                tests: testResults
            };
            if (codeCoverage) {
                result.codecoverage = yield this.getTestCodeCoverage();
                result.summary.orgWideCoverage = yield this.getOrgWideCoverage();
            }
            return result;
        });
    }
    getOrgWideCoverage() {
        return __awaiter(this, void 0, void 0, function* () {
            const orgWideCoverageResult = (yield this.connection.tooling.query('SELECT PercentCovered FROM ApexOrgWideCoverage'));
            if (orgWideCoverageResult.records.length === 0) {
                return '0%';
            }
            return `${orgWideCoverageResult.records[0].PercentCovered}%`;
        });
    }
    getTestCodeCoverage() {
        return __awaiter(this, void 0, void 0, function* () {
            const codeCoverageQuery = 'SELECT ApexClassOrTrigger.Id, ApexClassOrTrigger.Name, NumLinesCovered, NumLinesUncovered, Coverage FROM ApexCodeCoverageAggregate';
            const codeCoverageResuls = (yield this.connection.tooling.query(codeCoverageQuery));
            const coverageResults = codeCoverageResuls.records.map(item => {
                const totalLines = item.NumLinesCovered + item.NumLinesUncovered;
                const percentage = this.calculatePercentage(item.NumLinesCovered, totalLines);
                return {
                    apexId: item.ApexClassOrTrigger.Id,
                    name: item.ApexClassOrTrigger.Name,
                    type: item.ApexClassOrTrigger.Id.startsWith('01p')
                        ? 'ApexClass'
                        : 'ApexTrigger',
                    numLinesCovered: item.NumLinesCovered,
                    numLinesUncovered: item.NumLinesUncovered,
                    percentage,
                    coveredLines: item.Coverage.coveredLines,
                    uncoveredLines: item.Coverage.uncoveredLines
                };
            });
            return coverageResults;
        });
    }
}
exports.TestService = TestService;
//# sourceMappingURL=testService.js.map