"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const chai_1 = require("chai");
const fs = require("fs");
const readline = require("readline");
const sinon_1 = require("sinon");
const execute_1 = require("../../src/execute");
const i18n_1 = require("../../src/i18n");
const $$ = testSetup_1.testSetup();
describe('Apex Execute Tests', () => __awaiter(void 0, void 0, void 0, function* () {
    const testData = new testSetup_1.MockTestOrgData();
    let mockConnection;
    let sandboxStub;
    let fsStub;
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig()
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username
            })
        });
        sandboxStub.stub(fs, 'readFileSync').returns('System.assert(true);');
        fsStub = sandboxStub.stub(fs, 'existsSync').returns(true);
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should execute and display successful result in correct format', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexExecute = new execute_1.ExecuteService(mockConnection);
        const log = '47.0 APEX_CODE,DEBUG;APEX_PROFILING,INFO\nExecute Anonymous: System.assert(true);|EXECUTION_FINISHED\n';
        const execAnonResult = {
            column: -1,
            line: -1,
            compiled: 'true',
            compileProblem: '',
            exceptionMessage: '',
            exceptionStackTrace: '',
            success: 'true'
        };
        const soapResponse = {
            'soapenv:Envelope': {
                'soapenv:Header': { DebuggingInfo: { debugLog: log } },
                'soapenv:Body': {
                    executeAnonymousResponse: { result: execAnonResult }
                }
            }
        };
        const expectedResult = {
            compiled: true,
            success: true,
            logs: log
        };
        sandboxStub
            .stub(execute_1.ExecuteService.prototype, 'connectionRequest')
            .resolves(soapResponse);
        const response = yield apexExecute.executeAnonymous({
            apexFilePath: 'filepath/to/anonApex/file'
        });
        chai_1.expect(response).to.eql(expectedResult);
    }));
    it('should execute and display runtime issue in correct format', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexExecute = new execute_1.ExecuteService(mockConnection);
        const log = '47.0 APEX_CODE,DEBUG;APEX_PROFILING,INFO\nExecute Anonymous: System.assert(false);|EXECUTION_FINISHED\n';
        const execAnonResult = {
            column: 1,
            line: 6,
            compiled: 'true',
            compileProblem: '',
            exceptionMessage: 'System.AssertException: Assertion Failed',
            exceptionStackTrace: 'AnonymousBlock: line 1, column 1',
            success: 'false'
        };
        const soapResponse = {
            'soapenv:Envelope': {
                'soapenv:Header': { DebuggingInfo: { debugLog: log } },
                'soapenv:Body': {
                    executeAnonymousResponse: { result: execAnonResult }
                }
            }
        };
        const expectedResult = {
            compiled: true,
            success: false,
            logs: log,
            diagnostic: [
                {
                    exceptionMessage: 'System.AssertException: Assertion Failed',
                    exceptionStackTrace: 'AnonymousBlock: line 1, column 1',
                    compileProblem: '',
                    columnNumber: 1,
                    lineNumber: 6
                }
            ]
        };
        sandboxStub
            .stub(execute_1.ExecuteService.prototype, 'connectionRequest')
            .resolves(soapResponse);
        const response = yield apexExecute.executeAnonymous({
            apexFilePath: 'filepath/to/anonApex/file'
        });
        chai_1.expect(response).to.eql(expectedResult);
    }));
    it('should execute and display compile issue in correct format', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexExecute = new execute_1.ExecuteService(mockConnection);
        const execAnonResult = {
            column: 1,
            line: 6,
            compiled: 'false',
            compileProblem: `Unexpected token '('.`,
            exceptionMessage: '',
            exceptionStackTrace: '',
            success: 'false'
        };
        const soapResponse = {
            'soapenv:Envelope': {
                'soapenv:Header': { DebuggingInfo: { debugLog: '' } },
                'soapenv:Body': {
                    executeAnonymousResponse: { result: execAnonResult }
                }
            }
        };
        const expectedResult = {
            compiled: false,
            success: false,
            logs: '',
            diagnostic: [
                {
                    columnNumber: 1,
                    lineNumber: 6,
                    compileProblem: `Unexpected token '('.`,
                    exceptionMessage: '',
                    exceptionStackTrace: ''
                }
            ]
        };
        sandboxStub
            .stub(execute_1.ExecuteService.prototype, 'connectionRequest')
            .resolves(soapResponse);
        const response = yield apexExecute.executeAnonymous({
            apexFilePath: 'filepath/to/anonApex/file'
        });
        chai_1.expect(response).to.eql(expectedResult);
    }));
    it('should handle access token session id error correctly', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexExecute = new execute_1.ExecuteService(mockConnection);
        const log = '47.0 APEX_CODE,DEBUG;APEX_PROFILING,INFO\nExecute Anonymous: System.assert(true);|EXECUTION_FINISHED\n';
        const execAnonResult = {
            column: -1,
            line: -1,
            compiled: 'true',
            compileProblem: '',
            exceptionMessage: '',
            exceptionStackTrace: '',
            success: 'true'
        };
        const soapResponse = {
            'soapenv:Envelope': {
                'soapenv:Header': { DebuggingInfo: { debugLog: log } },
                'soapenv:Body': {
                    executeAnonymousResponse: { result: execAnonResult }
                }
            }
        };
        const expectedResult = {
            compiled: true,
            success: true,
            logs: log
        };
        const connRequestStub = sandboxStub.stub(execute_1.ExecuteService.prototype, 'connectionRequest');
        sandboxStub.stub(execute_1.ExecuteService.prototype, 'refreshAuth');
        const error = new Error('INVALID_SESSION_ID');
        error.name = 'ERROR_HTTP_500';
        connRequestStub.onFirstCall().throws(error);
        connRequestStub.onSecondCall().resolves(soapResponse);
        const response = yield apexExecute.executeAnonymous({
            apexFilePath: 'filepath/to/anonApex/file'
        });
        chai_1.expect(response).to.eql(expectedResult);
        chai_1.expect(connRequestStub.calledTwice);
    }));
    it('should raise an error when the source file is not found', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexFile = 'filepath/to/anonApex/file';
        const apexExecute = new execute_1.ExecuteService(mockConnection);
        fsStub.restore();
        fsStub.returns(false);
        try {
            yield apexExecute.executeAnonymous({ apexFilePath: apexFile });
            chai_1.assert.fail('Expected an error');
        }
        catch (e) {
            chai_1.assert.equal(i18n_1.nls.localize('file_not_found_error', apexFile), e.message);
        }
    }));
    it('should handle Buffer input correctly', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexExecute = new execute_1.ExecuteService(mockConnection);
        const log = '47.0 APEX_CODE,DEBUG;APEX_PROFILING,INFO\nExecute Anonymous: System.assert(true);|EXECUTION_FINISHED\n';
        const bufferInput = Buffer.from('System.assert(true);');
        const execAnonResult = {
            column: -1,
            line: -1,
            compiled: 'true',
            compileProblem: '',
            exceptionMessage: '',
            exceptionStackTrace: '',
            success: 'true'
        };
        const soapResponse = {
            'soapenv:Envelope': {
                'soapenv:Header': { DebuggingInfo: { debugLog: log } },
                'soapenv:Body': {
                    executeAnonymousResponse: { result: execAnonResult }
                }
            }
        };
        const expectedResult = {
            compiled: true,
            success: true,
            logs: log
        };
        sandboxStub
            .stub(execute_1.ExecuteService.prototype, 'connectionRequest')
            .resolves(soapResponse);
        const response = yield apexExecute.executeAnonymous({
            apexCode: bufferInput
        });
        chai_1.expect(response).to.eql(expectedResult);
    }));
    it('should throw an error if no option is specified', () => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const executeService = new execute_1.ExecuteService(mockConnection);
            yield executeService.executeAnonymous({});
            chai_1.assert.fail();
        }
        catch (e) {
            chai_1.assert.equal(i18n_1.nls.localize('option_exec_anon_error'), e.message);
        }
    }));
    it('should throw an error if user input fails', () => __awaiter(void 0, void 0, void 0, function* () {
        const errorText = 'This is the error';
        // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
        const on = (event, listener) => {
            try {
                if (event === 'error') {
                    listener(new Error(errorText));
                }
                listener();
            }
            catch (e) {
                throw e;
            }
        };
        sandboxStub
            .stub(readline, 'createInterface')
            //@ts-ignore
            .returns({ on });
        try {
            const executeService = new execute_1.ExecuteService(mockConnection);
            yield executeService.getUserInput();
        }
        catch (e) {
            chai_1.assert.equal(i18n_1.nls.localize('unexpected_exec_anon_input_error', errorText), e.message);
        }
    }));
    it('should process user input correctly', () => __awaiter(void 0, void 0, void 0, function* () {
        const inputText = 'This should be the only text';
        // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
        const on = (event, listener) => {
            listener(inputText);
        };
        sandboxStub
            .stub(readline, 'createInterface')
            //@ts-ignore
            .returns({ on });
        const executeService = new execute_1.ExecuteService(mockConnection);
        const text = yield executeService.getUserInput();
        chai_1.expect(text).to.equal(`${inputText}\n`);
    }));
    it('should throw error if user is idle', () => __awaiter(void 0, void 0, void 0, function* () {
        // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
        const on = (event, listener) => {
            listener();
        };
        sandboxStub
            .stub(readline, 'createInterface')
            //@ts-ignore
            .returns({ on });
        try {
            const executeService = new execute_1.ExecuteService(mockConnection);
            yield executeService.getUserInput();
        }
        catch (e) {
            chai_1.assert.equal(i18n_1.nls.localize('exec_anon_input_timeout'), e.message);
        }
    }));
}));
//# sourceMappingURL=executeService.test.js.map