"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const chai_1 = require("chai");
const fs = require("fs");
const sinon_1 = require("sinon");
const logService_1 = require("../../src/logs/logService");
const path = require("path");
const stream = require("stream");
const $$ = testSetup_1.testSetup();
describe('Apex Log Service Tests', () => {
    const testData = new testSetup_1.MockTestOrgData();
    let mockConnection;
    let sandboxStub;
    let toolingRequestStub;
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig()
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username
            })
        });
        toolingRequestStub = sandboxStub.stub(logService_1.LogService.prototype, 'toolingRequest');
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should return correct number of logs', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexLogGet = new logService_1.LogService(mockConnection);
        const logs = ['07WgsWfsFF', 'FTWrd5lfg'];
        const ids = [{ Id: '48jnskd' }, { Id: '67knmdfklDF' }];
        const queryRecords = { records: ids };
        toolingRequestStub.onFirstCall().resolves(logs[0]);
        toolingRequestStub.onSecondCall().resolves(logs[1]);
        const toolingQueryStub = sandboxStub.stub(mockConnection.tooling, 'query');
        //@ts-ignore
        toolingQueryStub.onFirstCall().resolves(queryRecords);
        const response = yield apexLogGet.getLogs({ numberOfLogs: 2 });
        chai_1.expect(response.length).to.eql(2);
    }));
    it('should return correct log given log id', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexLogGet = new logService_1.LogService(mockConnection);
        const log = '48.0 APEX_CODE,FINEST;APEX_PROFILING,INFO;CALLOUT..';
        const getLogIdStub = sandboxStub.stub(logService_1.LogService.prototype, 'getLogIds');
        toolingRequestStub.onFirstCall().resolves(log);
        const response = yield apexLogGet.getLogs({ logId: '07L5w00005PGdTnEAL' });
        chai_1.expect(response.length).to.eql(1);
        chai_1.expect(getLogIdStub.callCount).to.eql(0);
    }));
    it('should handle exceeding log limit', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexLogGet = new logService_1.LogService(mockConnection);
        const ids = [
            { Id: '48jnskd' },
            { Id: '67knmdfklDF' },
            { Id: 'CODE' },
            { Id: 'FINEST' },
            { Id: 'PROFILING' },
            { Id: 'INFO' },
            { Id: 'CALLOUT' },
            { Id: 'ASDA' },
            { Id: 'ADAD' },
            { Id: 'Ajkl' },
            { Id: 'SADkl' },
            { Id: 'FSDFS' },
            { Id: 'DASD' },
            { Id: 'ASD' },
            { Id: 'NKJN' },
            { Id: 'ADA' },
            { Id: 'GGS' },
            { Id: 'ADASD' },
            { Id: 'SDA' },
            { Id: 'ADA' },
            { Id: 'JKH' },
            { Id: 'DH' },
            { Id: 'FGFD' },
            { Id: 'SFSDF' },
            { Id: 'DSASD' }
        ];
        const queryRecords = { records: ids };
        const toolingQueryStub = sandboxStub.stub(mockConnection.tooling, 'query');
        //@ts-ignore
        toolingQueryStub.onFirstCall().resolves(queryRecords);
        const response = yield apexLogGet.getLogs({ numberOfLogs: 27 });
        chai_1.expect(response.length).to.eql(25);
    }));
    it('should handle invalid id', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexLogGet = new logService_1.LogService(mockConnection);
        toolingRequestStub.throws(new Error('invalid id'));
        try {
            yield apexLogGet.getLogs({ logId: '07L5tgg0005PGdTnEAL' });
            chai_1.assert.fail();
        }
        catch (e) {
            chai_1.expect(e.message).to.equal('invalid id');
        }
    }));
    it('should throw an error if 0 logs are requested', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexLogGet = new logService_1.LogService(mockConnection);
        try {
            yield apexLogGet.getLogs({ numberOfLogs: 0 });
            chai_1.assert.fail();
        }
        catch (e) {
            chai_1.expect(e.message).to.equal('Expected number of logs to be greater than 0.');
        }
    }));
    it('should store logs in the directory', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexLogGet = new logService_1.LogService(mockConnection);
        const filePath = path.join('testTmp', 'file', 'path', 'logs');
        const logIds = ['07L5tgg0005PGdTnEAL', '07L5tgg0005PGdTnFPL'];
        sandboxStub.stub(logService_1.LogService.prototype, 'getLogIds').resolves(logIds);
        const createStreamStub = sandboxStub.stub(fs, 'createWriteStream');
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        createStreamStub.onCall(0).returns(new stream.PassThrough());
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        createStreamStub.onCall(1).returns(new stream.PassThrough());
        sandboxStub.stub(fs, 'closeSync');
        sandboxStub.stub(fs, 'openSync');
        const logs = ['48jnskd', '57fskjf'];
        toolingRequestStub.onFirstCall().resolves(logs[0]);
        toolingRequestStub.onSecondCall().resolves(logs[1]);
        const response = yield apexLogGet.getLogs({
            numberOfLogs: 2,
            outputDir: filePath
        });
        chai_1.expect(response).to.deep.equal([
            path.join(filePath, '07L5tgg0005PGdTnEAL.log'),
            path.join(filePath, '07L5tgg0005PGdTnFPL.log')
        ]);
        chai_1.expect(createStreamStub.callCount).to.eql(2);
    }));
    it('should successfully create a .log file', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexLogGet = new logService_1.LogService(mockConnection);
        const filePath = path.join('path', 'to', 'logs');
        const logIds = ['07WgsWfad'];
        const logs = ['log content'];
        const logsPath = path.join(filePath, `${logIds[0]}.log`);
        sandboxStub.stub(fs, 'existsSync').returns(true);
        const createStreamStub = sandboxStub.stub(fs, 'createWriteStream');
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        createStreamStub.onCall(0).returns(new stream.PassThrough());
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        createStreamStub.onCall(1).returns(new stream.PassThrough());
        sandboxStub.stub(fs, 'closeSync');
        sandboxStub.stub(fs, 'openSync');
        toolingRequestStub.onFirstCall().resolves(logs[0]);
        toolingRequestStub.onSecondCall().resolves(logs[1]);
        yield apexLogGet.getLogs({
            logId: '07WgsWfad',
            outputDir: filePath
        });
        chai_1.expect(createStreamStub.calledWith(logsPath)).to.be.true;
    }));
    it('should throw an error if numberOfLogs or logId are not given', () => __awaiter(void 0, void 0, void 0, function* () {
        const apexLogGet = new logService_1.LogService(mockConnection);
        const filePath = path.join('path', 'to', 'logs');
        try {
            yield apexLogGet.getLogs({ outputDir: filePath });
            chai_1.assert.fail();
        }
        catch (e) {
            chai_1.expect(e.message).to.equal('To retrieve logs, specify the log ID or the number of logs.');
        }
    }));
});
//# sourceMappingURL=logService.test.js.map