"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const sinon_1 = require("sinon");
const streaming_1 = require("../../src/streaming");
const chai_1 = require("chai");
const faye_1 = require("faye");
const assert_1 = require("assert");
const i18n_1 = require("../../src/i18n");
const $$ = testSetup_1.testSetup();
let mockConnection;
let sandboxStub;
const testData = new testSetup_1.MockTestOrgData();
const testResultMsg = {
    event: {
        createdDate: '2020-08-03T22:58:58.000+0000',
        type: 'updated'
    },
    sobject: {
        Id: '707xx0000AGQ3jbQQD'
    }
};
describe('Streaming API Client', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig()
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username
            })
        });
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should build a valid streaming url', () => {
        const streamClient = new streaming_1.StreamingClient(mockConnection);
        const result = streamClient.getStreamURL('https://na1.salesforce.com/');
        chai_1.expect(result).to.equal('https://na1.salesforce.com/cometd/36.0');
    });
    it('should initialize Faye Client', () => {
        const stubOn = sandboxStub.stub(faye_1.Client.prototype, 'on');
        const stubAddExtension = sandboxStub.stub(faye_1.Client.prototype, 'addExtension');
        new streaming_1.StreamingClient(mockConnection);
        chai_1.expect(stubOn.calledTwice).to.equal(true);
        chai_1.expect(stubOn.getCall(0).args[0]).to.equal('transport:up');
        chai_1.expect(stubOn.getCall(1).args[0]).to.equal('transport:down');
        chai_1.expect(stubAddExtension.calledOnce).to.equal(true);
    });
    it('should initialize Faye Client header', () => __awaiter(void 0, void 0, void 0, function* () {
        const stubSetHeader = sandboxStub.stub(faye_1.Client.prototype, 'setHeader');
        const streamClient = new streaming_1.StreamingClient(mockConnection);
        yield streamClient.init();
        chai_1.expect(stubSetHeader.calledOnce).to.equal(true);
        chai_1.expect(stubSetHeader.getCall(0).args[0]).to.equal('Authorization');
        const accessToken = mockConnection.getConnectionOptions().accessToken;
        chai_1.expect(stubSetHeader.getCall(0).args[1]).to.equal(`OAuth ${accessToken}`);
    }));
    it('should initialize Faye Client throws authentication error', () => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub.stub(core_1.Connection.prototype, 'getConnectionOptions').returns({
            accessToken: undefined
        });
        const streamClient = new streaming_1.StreamingClient(mockConnection);
        try {
            yield streamClient.init();
            assert_1.fail('Test should have thrown an error');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal(i18n_1.nls.localize('no_access_token_found'));
        }
    }));
    it('should disconect when subscribe throws an error', () => __awaiter(void 0, void 0, void 0, function* () {
        const stubSubscribe = sandboxStub
            .stub(faye_1.Client.prototype, 'subscribe')
            .throwsException('custom subscribe error');
        const stubDisconnect = sandboxStub.stub(faye_1.Client.prototype, 'disconnect');
        const streamClient = new streaming_1.StreamingClient(mockConnection);
        try {
            yield streamClient.subscribe('707xx0000AGQ3jbQQD');
            assert_1.fail('Test should have thrown an error');
        }
        catch (e) {
            chai_1.expect(stubSubscribe.calledOnce).to.equal(true);
            chai_1.expect(e.name).to.equal('custom subscribe error');
            chai_1.expect(stubDisconnect.calledOnce).to.equal(true);
        }
    }));
    it('should throw error if handler can not find test records', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        mockToolingQuery.resolves({
            done: true,
            totalSize: 0,
            records: []
        });
        const streamClient = new streaming_1.StreamingClient(mockConnection);
        try {
            streamClient.subscribedTestRunId = '707xx0000AGQ3jbQQD';
            yield streamClient.handler(testResultMsg);
            assert_1.fail('Test should have thrown an error');
        }
        catch (e) {
            chai_1.expect(mockToolingQuery.calledOnce).to.equal(true);
            chai_1.expect(mockToolingQuery.getCall(0).args[0]).to.equal(`SELECT Id, Status, ApexClassId, TestRunResultId FROM ApexTestQueueItem WHERE ParentJobId = '${testResultMsg.sobject.Id}'`);
            chai_1.expect(e.message).to.equal(i18n_1.nls.localize('no_test_queue_results', testResultMsg.sobject.Id));
        }
    }));
    it('should not run a query if the subscribed test run id does not match the message test run id', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        const streamClient = new streaming_1.StreamingClient(mockConnection);
        streamClient.subscribedTestRunId = '707xx0000gtQ3jx3x5';
        const streamHandlerResult = yield streamClient.handler(testResultMsg);
        chai_1.expect(mockToolingQuery.calledOnce).to.equal(false);
        chai_1.expect(streamHandlerResult).to.equal(null);
    }));
    it('should return ApexTestQueueItem records from handler function', () => __awaiter(void 0, void 0, void 0, function* () {
        const queryResponse = {
            done: true,
            totalSize: 1,
            records: [
                {
                    Id: '7092M000000Vt94QAC',
                    Status: "Completed" /* Completed */,
                    ApexClassId: '01p2M00000O6tXZQAZ',
                    TestRunResultId: '05m2M000000TgYuQAK'
                }
            ]
        };
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        mockToolingQuery.resolves(queryResponse);
        const streamClient = new streaming_1.StreamingClient(mockConnection);
        streamClient.subscribedTestRunId = '707xx0000AGQ3jbQQD';
        const results = yield streamClient.handler(testResultMsg);
        chai_1.expect(mockToolingQuery.calledOnce).to.equal(true);
        chai_1.expect(results).to.deep.equal(queryResponse);
    }));
    it('should return no results when a ApexTestQueueItem record has a pending status', () => __awaiter(void 0, void 0, void 0, function* () {
        const queryResponse = {
            done: true,
            totalSize: 2,
            records: [
                {
                    Id: '7092M000000Vt94QAC',
                    Status: "Completed" /* Completed */,
                    ApexClassId: '01p2M00000O6tXZQAZ',
                    TestRunResultId: '05m2M000000TgYuQAK'
                },
                {
                    Id: '709xx000000Vt94QAD',
                    Status: "Processing" /* Processing */,
                    ApexClassId: '01pxx00000O6tXZQAx',
                    TestRunResultId: '05mxx000000TgYuQAw'
                }
            ]
        };
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        mockToolingQuery.resolves(queryResponse);
        const streamClient = new streaming_1.StreamingClient(mockConnection);
        streamClient.subscribedTestRunId = '707xx0000AGQ3jbQQD';
        const results = yield streamClient.handler(testResultMsg);
        chai_1.expect(mockToolingQuery.calledOnce).to.equal(true);
        chai_1.expect(results).to.equal(null);
    }));
});
//# sourceMappingURL=streamingClient.test.js.map