"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const tests_1 = require("../../src/tests");
const testData_1 = require("./testData");
const $$ = testSetup_1.testSetup();
let mockConnection;
let sandboxStub;
let toolingQueryStub;
const testData = new testSetup_1.MockTestOrgData();
describe('Run Apex tests code coverage', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig()
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username
            })
        });
        toolingQueryStub = sandboxStub.stub(mockConnection.tooling, 'query');
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should return org wide coverage result', () => __awaiter(void 0, void 0, void 0, function* () {
        toolingQueryStub.onFirstCall().resolves({
            done: true,
            totalSize: 1,
            records: [
                {
                    PercentCovered: '33'
                }
            ]
        });
        const testSrv = new tests_1.TestService(mockConnection);
        const orgWideCoverageResult = yield testSrv.getOrgWideCoverage();
        chai_1.expect(orgWideCoverageResult).to.equal('33%');
    }));
    it('should return 0% org wide coverage when no records are available', () => __awaiter(void 0, void 0, void 0, function* () {
        toolingQueryStub.onFirstCall().resolves({
            done: true,
            totalSize: 0,
            records: []
        });
        const testSrv = new tests_1.TestService(mockConnection);
        const orgWideCoverageResult = yield testSrv.getOrgWideCoverage();
        chai_1.expect(orgWideCoverageResult).to.equal('0%');
        chai_1.expect(toolingQueryStub.getCall(0).args[0]).to.equal('SELECT PercentCovered FROM ApexOrgWideCoverage');
    }));
    it('should return test code coverage result', () => __awaiter(void 0, void 0, void 0, function* () {
        toolingQueryStub.resolves({
            done: true,
            totalSize: 3,
            records: testData_1.codeCoverageQueryResult
        });
        const testSrv = new tests_1.TestService(mockConnection);
        const testCodeCoverageResult = yield testSrv.getTestCodeCoverage();
        chai_1.expect(testCodeCoverageResult.length).to.equal(3);
        chai_1.expect(toolingQueryStub.getCall(0).args[0]).to.equal('SELECT ApexClassOrTrigger.Id, ApexClassOrTrigger.Name, NumLinesCovered, NumLinesUncovered, Coverage FROM ApexCodeCoverageAggregate');
        chai_1.expect(testCodeCoverageResult[0].apexId).to.equal('01pxx00000avcNeAAL');
        chai_1.expect(testCodeCoverageResult[0].name).to.equal('ApexClassExample');
        chai_1.expect(testCodeCoverageResult[0].type).to.equal('ApexClass');
        chai_1.expect(testCodeCoverageResult[0].numLinesCovered).to.equal(0);
        chai_1.expect(testCodeCoverageResult[0].numLinesUncovered).to.equal(9);
        chai_1.expect(testCodeCoverageResult[0].percentage).to.equal('0%');
        chai_1.expect(testCodeCoverageResult[1].apexId).to.equal('01pxx00000avc00AAL');
        chai_1.expect(testCodeCoverageResult[1].name).to.equal('ApexSampleV2');
        chai_1.expect(testCodeCoverageResult[1].type).to.equal('ApexClass');
        chai_1.expect(testCodeCoverageResult[1].numLinesCovered).to.equal(19);
        chai_1.expect(testCodeCoverageResult[1].numLinesUncovered).to.equal(1);
        chai_1.expect(testCodeCoverageResult[1].percentage).to.equal('95%');
        chai_1.expect(testCodeCoverageResult[2].apexId).to.equal('01qxp00000av340AAL');
        chai_1.expect(testCodeCoverageResult[2].name).to.equal('MyTestTrigger');
        chai_1.expect(testCodeCoverageResult[2].type).to.equal('ApexTrigger');
        chai_1.expect(testCodeCoverageResult[2].numLinesCovered).to.equal(0);
        chai_1.expect(testCodeCoverageResult[2].numLinesUncovered).to.equal(0);
        chai_1.expect(testCodeCoverageResult[2].percentage).to.equal('0%');
    }));
    it('should return test code coverage result', () => __awaiter(void 0, void 0, void 0, function* () {
        toolingQueryStub.resolves({
            done: true,
            totalSize: 0,
            records: []
        });
        const testSrv = new tests_1.TestService(mockConnection);
        const testCodeCoverageResult = yield testSrv.getTestCodeCoverage();
        chai_1.expect(testCodeCoverageResult.length).to.equal(0);
    }));
});
//# sourceMappingURL=codeCoverage.test.js.map