"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const tests_1 = require("../../src/tests");
const streaming_1 = require("../../src/streaming");
const assert_1 = require("assert");
const i18n_1 = require("../../src/i18n");
const testData_1 = require("./testData");
const $$ = testSetup_1.testSetup();
let mockConnection;
let sandboxStub;
let toolingRequestStub;
const testData = new testSetup_1.MockTestOrgData();
describe('Run Apex tests synchronously', () => {
    let testRequest = {};
    const requestOptions = {
        tests: [{ className: 'TestSample' }],
        maxFailedTests: 2,
        testLevel: 'RunSpecifiedTests'
    };
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig()
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username
            })
        });
        toolingRequestStub = sandboxStub.stub(mockConnection.tooling, 'request');
        testRequest = {
            method: 'POST',
            url: `${mockConnection.tooling._baseUrl()}/runTestsSynchronous`,
            body: JSON.stringify(requestOptions),
            headers: { 'content-type': 'application/json' }
        };
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should run a successful test', () => __awaiter(void 0, void 0, void 0, function* () {
        const requestResult = {
            apexLogId: '07Lxx00000cxy6YUAQ',
            failures: [],
            numFailures: 0,
            numTestsRun: 1,
            successes: [
                {
                    id: '01pxx00000NWwb3AAD',
                    methodName: 'testOne',
                    name: 'TestSample',
                    namespace: null,
                    seeAllData: false,
                    time: 107
                }
            ],
            totalTime: 270
        };
        toolingRequestStub.withArgs(testRequest).returns(requestResult);
        const testSrv = new tests_1.TestService(mockConnection);
        const testResult = yield testSrv.runTestSynchronous(requestOptions);
        chai_1.expect(testResult).to.be.a('object');
        chai_1.expect(toolingRequestStub.calledOnce).to.equal(true);
        chai_1.expect(testResult).to.deep.equals(requestResult);
    }));
    it('should run a test with failures', () => __awaiter(void 0, void 0, void 0, function* () {
        const requestResult = [
            {
                message: "This class name's value is invalid: TestConfig. Provide the name of an Apex class that has test methods.",
                errorCode: 'INVALID_INPUT'
            }
        ];
        toolingRequestStub.withArgs(testRequest).returns(requestResult);
        const testSrv = new tests_1.TestService(mockConnection);
        const testResult = yield testSrv.runTestSynchronous(requestOptions);
        chai_1.expect(testResult).to.be.a('array');
        chai_1.expect(toolingRequestStub.calledOnce).to.equal(true);
        chai_1.expect(testResult).to.deep.equals(requestResult);
    }));
});
describe('Run Apex tests asynchronously', () => {
    const pollResponse = {
        done: true,
        totalSize: 1,
        records: [
            {
                Id: '7092M000000Vt94QAC',
                Status: "Completed" /* Completed */,
                ApexClassId: '01p2M00000O6tXZQAZ',
                TestRunResultId: '05m2M000000TgYuQAK'
            }
        ]
    };
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        $$.setConfigStubContents('AuthInfoConfig', {
            contents: yield testData.getConfig()
        });
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username
            })
        });
        testData_1.testResultData.summary.orgId = mockConnection.getAuthInfoFields().orgId;
        testData_1.testResultData.summary.username = mockConnection.getUsername();
        toolingRequestStub = sandboxStub.stub(mockConnection.tooling, 'request');
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('should run a successful test', () => __awaiter(void 0, void 0, void 0, function* () {
        const requestOptions = {
            classNames: 'TestSample',
            testLevel: "RunSpecifiedTests" /* RunSpecifiedTests */
        };
        const testAsyncRequest = {
            method: 'POST',
            url: `${mockConnection.tooling._baseUrl()}/runTestsAsynchronous`,
            body: JSON.stringify(requestOptions),
            headers: { 'content-type': 'application/json' }
        };
        toolingRequestStub.withArgs(testAsyncRequest).returns(testData_1.testRunId);
        sandboxStub
            .stub(streaming_1.StreamingClient.prototype, 'subscribe')
            .resolves(pollResponse);
        const testSrv = new tests_1.TestService(mockConnection);
        const mockTestResultData = sandboxStub
            .stub(testSrv, 'getTestResultData')
            .resolves(testData_1.testResultData);
        const testResult = yield testSrv.runTestAsynchronous(requestOptions);
        chai_1.expect(testResult).to.be.a('object');
        chai_1.expect(toolingRequestStub.calledOnce).to.equal(true);
        chai_1.expect(mockTestResultData.calledOnce).to.equal(true);
        chai_1.expect(mockTestResultData.getCall(0).args[0]).to.equal(pollResponse);
        chai_1.expect(mockTestResultData.getCall(0).args[1]).to.equal(testData_1.testRunId);
        chai_1.expect(testResult).to.equal(testData_1.testResultData);
    }));
    it('should throw an error on refresh token issue', () => __awaiter(void 0, void 0, void 0, function* () {
        const requestOptions = {
            classNames: 'TestSample',
            testLevel: "RunSpecifiedTests" /* RunSpecifiedTests */
        };
        sandboxStub
            .stub(streaming_1.StreamingClient.prototype, 'init')
            .throwsException('No access token');
        const testSrv = new tests_1.TestService(mockConnection);
        try {
            yield testSrv.runTestAsynchronous(requestOptions);
            assert_1.fail('Test should have thrown an error');
        }
        catch (e) {
            chai_1.expect(e.name).to.equal('No access token');
        }
    }));
    it('should return formatted test results', () => __awaiter(void 0, void 0, void 0, function* () {
        const testSrv = new tests_1.TestService(mockConnection);
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        mockToolingQuery.onFirstCall().resolves({
            done: true,
            totalSize: 1,
            records: [
                {
                    AsyncApexJobId: testData_1.testRunId,
                    Status: "Completed" /* Completed */,
                    StartTime: '2020-07-12T02:54:47.000+0000',
                    TestTime: 1765,
                    UserId: '005xx000000abcDAAU'
                }
            ]
        });
        mockToolingQuery.onSecondCall().resolves({
            done: true,
            totalSize: 1,
            records: [
                {
                    Id: '07Mxx00000F2Xx6UAF',
                    QueueItemId: '7092M000000Vt94QAC',
                    StackTrace: null,
                    Message: null,
                    AsyncApexJobId: testData_1.testRunId,
                    MethodName: 'testLoggerLog',
                    Outcome: "Pass" /* Pass */,
                    ApexLogId: null,
                    ApexClass: {
                        Id: '01pxx00000O6tXZQAZ',
                        Name: 'TestLogger',
                        NamespacePrefix: 't3st',
                        FullName: 't3st__TestLogger'
                    },
                    RunTime: 8,
                    TestTimestamp: '3'
                }
            ]
        });
        const getTestResultData = yield testSrv.getTestResultData(pollResponse, testData_1.testRunId);
        let summaryQuery = 'SELECT AsyncApexJobId, Status, ClassesCompleted, ClassesEnqueued, ';
        summaryQuery += 'MethodsEnqueued, StartTime, EndTime, TestTime, UserId ';
        summaryQuery += `FROM ApexTestRunResult WHERE AsyncApexJobId = '${testData_1.testRunId}'`;
        chai_1.expect(mockToolingQuery.getCall(0).args[0]).to.equal(summaryQuery);
        let testResultQuery = 'SELECT Id, QueueItemId, StackTrace, Message, ';
        testResultQuery +=
            'RunTime, TestTimestamp, AsyncApexJobId, MethodName, Outcome, ApexLogId, ';
        testResultQuery +=
            'ApexClass.Id, ApexClass.Name, ApexClass.NamespacePrefix, ApexClass.FullName ';
        testResultQuery += `FROM ApexTestResult WHERE QueueItemId IN ('${pollResponse.records[0].Id}')`;
        chai_1.expect(mockToolingQuery.getCall(1).args[0]).to.equal(testResultQuery);
        chai_1.expect(getTestResultData).to.deep.equals(testData_1.testResultData);
    }));
    it('should return an error if no test results are found', () => __awaiter(void 0, void 0, void 0, function* () {
        const testSrv = new tests_1.TestService(mockConnection);
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        mockToolingQuery.onFirstCall().resolves({
            done: true,
            totalSize: 0,
            records: []
        });
        try {
            yield testSrv.getTestResultData(pollResponse, testData_1.testRunId);
            assert_1.fail('Test should have thrown an error');
        }
        catch (e) {
            chai_1.expect(e.message).to.equal(i18n_1.nls.localize('no_test_result_summary', testData_1.testRunId));
        }
    }));
    it('should return formatted test results with code coverage', () => __awaiter(void 0, void 0, void 0, function* () {
        const testSrv = new tests_1.TestService(mockConnection);
        const mockToolingQuery = sandboxStub.stub(mockConnection.tooling, 'query');
        mockToolingQuery.onCall(0).resolves({
            done: true,
            totalSize: 1,
            records: [
                {
                    AsyncApexJobId: testData_1.testRunId,
                    Status: "Completed" /* Completed */,
                    StartTime: '2020-07-12T02:54:47.000+0000',
                    TestTime: 1765,
                    UserId: '005xx000000abcDAAU'
                }
            ]
        });
        mockToolingQuery.onCall(1).resolves({
            done: true,
            totalSize: 6,
            records: testData_1.mixedTestResults
        });
        mockToolingQuery.onCall(2).resolves({
            done: true,
            totalSize: 3,
            records: testData_1.codeCoverageQueryResult
        });
        mockToolingQuery.onCall(3).resolves({
            done: true,
            totalSize: 1,
            records: [
                {
                    PercentCovered: '57'
                }
            ]
        });
        const getTestResultData = yield testSrv.getTestResultData(pollResponse, testData_1.testRunId, true);
        // verify summary data
        chai_1.expect(getTestResultData.summary.failRate).to.equal('33%');
        chai_1.expect(getTestResultData.summary.numTestsRan).to.equal(6);
        chai_1.expect(getTestResultData.summary.orgId).to.equal(mockConnection.getAuthInfoFields().orgId);
        chai_1.expect(getTestResultData.summary.outcome).to.equal('Completed');
        chai_1.expect(getTestResultData.summary.passRate).to.equal('50%');
        chai_1.expect(getTestResultData.summary.skipRate).to.equal('17%');
        chai_1.expect(getTestResultData.summary.username).to.equal(mockConnection.getUsername());
        chai_1.expect(getTestResultData.summary.orgWideCoverage).to.equal('57%');
        chai_1.expect(getTestResultData.tests.length).to.equal(6);
        chai_1.expect(getTestResultData.codecoverage.length).to.equal(3);
    }));
});
//# sourceMappingURL=testService.test.js.map